module plantypes

//open util/ordering[Plan] as aplan
open util/ordering[Time] as atime
//open util/time
open  util/boolean

sig Name{}

sig Time{}

sig Plan {
	disj name, type, description: one Name, priority: one Int,	
}

sig Role {
	name:one Name,
	interacts_with: some SWS
}

sig SS_RegisterPlan extends Plan{
	advertises: some Interface 
}

sig SS_FinderPlan extends Plan {
	discovers: some Interface,
	interacts_with: some SS_MatchmakerAgent,
	finding_result: Bool
}

sig SS_AgreementPlan extends Plan{
	negotiates: some Interface,
	agreement_result: Bool
}

sig SS_ExecutorPlan extends Plan{
	executes: some Process,
	uses: some Grounding
}


sig SWA {
	plays: Role one-> Time,
	applies: some Plan 
}


sig SS_MatchmakerAgent {
	appliesSS_RegisterPlan :  some SS_RegisterPlan,
	playsRegistrationRole:  RegistrationRole one ->Time
}

sig RegistrationRole {
	advertises: some SWS 
}

some sig Interface{
	presents: some SWS
}

sig Process{
	described_by: some SWS
}

sig Grounding{
	supports: some SWS,
	calls: some WebService 
}

sig SWS{
	name: one Name,
	composed_of: some WebService,
}

sig Service{
	name:one Name
}

sig WebService extends Service{}


sig Environment{
	name: one Name,
	hasService: set Service,
}
fact ServiceComposition{
	all s:Service | s.~hasService != none
	all wb:WebService | wb.~composed_of != none
}
fact InterfaceControl{
  all f:SS_FinderPlan, r:SS_RegisterPlan, 
  a:SS_AgreementPlan |  
  f.discovers in 	r.advertises && 	
  a.negotiates in f.discovers 
  all i:Interface, p:Process, g:Grounding, e:SS_ExecutorPlan | 
	p in e.executes && g in e.uses &&
	p.described_by in i.presents 	&& g.supports in i.presents 
}
fact InheritanceBreak{
	no a:SWA,rp:SS_RegisterPlan| 	rp in a.applies
}

fact Agent_SWSPlanOrdering {
	all swa:SWA, sm:SS_MatchmakerAgent| 
	(SS_FinderPlan in swa.applies =>	
      #(sm.appliesSS_RegisterPlan) >=1)&&
	(SS_AgreementPlan in swa.applies  => 
      SS_FinderPlan  in 	swa.applies) &&
	(SS_ExecutorPlan  in swa.applies => 
      SS_AgreementPlan 	in swa.applies)
}
pred show{}

pred eBarter (BarterManager:SWA, BarterRole: Role, 	BarterService: SWS, BSInterface: Interface,BSGrounding: 	Grounding, BSProcess: Process, TradingService: WebService, 	Discover: SS_FinderPlan, 
Haggle:SS_AgreementPlan, 	Invoke:SS_ExecutorPlan ){ some t:Time |
 Discover in 	BarterManager.applies  && 
	Haggle in BarterManager.applies  && 
	Invoke in BarterManager.applies && BarterRole in BarterManager.plays.t  && 
	Discover.discovers = 	BSInterface&& 
	Haggle.negotiates = BSInterface 	&& Invoke.executes = BSProcess && 
	Invoke.uses=Grounding &&
	BSProcess.described_by=BarterService 	&& 	
	BSGrounding.supports =BarterService&&
	BSGrounding.calls=TradingService &&  
  BSInterface.presents=BarterService && 	
	BarterService.composed_of=TradingService


}


run  eBarter for 4 but exactly 1 RegistrationRole, 1 SWS, 1 Interface, 1 Process, 1 Grounding, 1 WebService,  1 Environment
